@@ -79,6 +79,7 @@ TEST_DEBUG = env("TEST_DEBUG", "0") == "1"
 SLOW_MACHINE = env("SLOW_MACHINE", "0") == "1"
 DEPRECATED_APIS = env("DEPRECATED_APIS", "0") == "1"
 TIMEOUT = int(env("TIMEOUT", 180 if SLOW_MACHINE else 60))
+EXPERIMENTAL_DUAL_FUND = env("EXPERIMENTAL_DUAL_FUND", "0") == "1"
 
 
 def wait_for(success, timeout=TIMEOUT):
@@ -283,11 +284,11 @@ class TailableProc(object):
                     if self.is_in_log(r):
                         print("({} was previously in logs!)".format(r))
                 raise TimeoutError('Unable to find "{}" in logs.'.format(exs))
-            elif not self.running:
-                raise ValueError('Process died while waiting for logs')
 
             with self.logs_cond:
                 if pos >= len(self.logs):
+                    if not self.running:
+                        raise ValueError('Process died while waiting for logs')
                     self.logs_cond.wait(1)
                     continue
 
@@ -600,7 +601,17 @@ class PrettyPrintingLightningRpc(LightningRpc):
     eyes. It has some overhead since we re-serialize the request and
     result to json in order to pretty print it.
 
+    Also validates (optional) schemas for us.
     """
+    def __init__(self, socket_path, executor=None, logger=logging,
+                 patch_json=True, jsonschemas={}):
+        super().__init__(
+            socket_path,
+            executor,
+            logger,
+            patch_json,
+        )
+        self.jsonschemas = jsonschemas
 
     def call(self, method, payload=None):
         id = self.next_id
@@ -614,6 +625,10 @@ class PrettyPrintingLightningRpc(LightningRpc):
             "id": id,
             "result": res
         }, indent=2))
+
+        if method in self.jsonschemas:
+            self.jsonschemas[method].validate(res)
+
         return res
 
 
@@ -624,6 +639,7 @@ class LightningNode(object):
                  allow_warning=False,
                  allow_bad_gossip=False,
                  db=None, port=None, disconnect=None, random_hsm=None, options=None,
+                 jsonschemas={},
                  **kwargs):
         self.bitcoin = bitcoind
         self.executor = executor
@@ -638,7 +654,7 @@ class LightningNode(object):
         self.rc = 0
 
         socket_path = os.path.join(lightning_dir, TEST_NETWORK, "lightning-rpc").format(node_id)
-        self.rpc = PrettyPrintingLightningRpc(socket_path, self.executor)
+        self.rpc = PrettyPrintingLightningRpc(socket_path, self.executor, jsonschemas=jsonschemas)
 
         self.daemon = LightningD(
             lightning_dir, bitcoindproxy=bitcoind.get_proxy(),
@@ -664,6 +680,8 @@ class LightningNode(object):
                 self.daemon.env["LIGHTNINGD_DEV_MEMLEAK"] = "1"
             if not may_reconnect:
                 self.daemon.opts["dev-no-reconnect"] = None
+        if EXPERIMENTAL_DUAL_FUND:
+            self.daemon.opts["experimental-dual-fund"] = None
 
         if options is not None:
             self.daemon.opts.update(options)
@@ -711,11 +729,12 @@ class LightningNode(object):
                 r'Funding tx {} depth'.format(fundingtx['txid']))
         return {'address': addr, 'wallettxid': wallettxid, 'fundingtx': fundingtx}
 
-    def fundwallet(self, sats, addrtype="p2sh-segwit"):
+    def fundwallet(self, sats, addrtype="p2sh-segwit", mine_block=True):
         addr = self.rpc.newaddr(addrtype)[addrtype]
         txid = self.bitcoin.rpc.sendtoaddress(addr, sats / 10**8)
-        self.bitcoin.generate_block(1)
-        self.daemon.wait_for_log('Owning output .* txid {} CONFIRMED'.format(txid))
+        if mine_block:
+            self.bitcoin.generate_block(1)
+            self.daemon.wait_for_log('Owning output .* txid {} CONFIRMED'.format(txid))
         return addr, txid
 
     def fundbalancedchannel(self, remote_node, total_capacity, announce=True):
@@ -735,6 +754,10 @@ class LightningNode(object):
             # expected to contribute that same amount
             chan_capacity = total_capacity // 2
             total_capacity = chan_capacity * 2
+            # Tell the node to equally dual-fund the channel
+            remote_node.rpc.call('funderupdate', {'policy': 'match',
+                                                  'policy_mod': 100,
+                                                  'fuzz_percent': 0})
         else:
             chan_capacity = total_capacity
 
@@ -742,18 +765,20 @@ class LightningNode(object):
 
         # Make sure the fundchannel is confirmed.
         num_tx = len(self.bitcoin.rpc.getrawmempool())
-        tx = self.rpc.fundchannel(remote_node.info['id'], chan_capacity, feerate='slow', minconf=0, announce=announce, push_msat=Millisatoshi(chan_capacity * 500))['tx']
+        res = self.rpc.fundchannel(remote_node.info['id'], chan_capacity, feerate='slow', minconf=0, announce=announce, push_msat=Millisatoshi(chan_capacity * 500))
         wait_for(lambda: len(self.bitcoin.rpc.getrawmempool()) == num_tx + 1)
-        self.bitcoin.generate_block(1)
+        blockid = self.bitcoin.generate_block(1)[0]
 
         # Generate the scid.
-        # NOTE This assumes only the coinbase and the fundchannel is
-        # confirmed in the block.
-        outnum = get_tx_p2wsh_outnum(self.bitcoin, tx, total_capacity)
+        outnum = get_tx_p2wsh_outnum(self.bitcoin, res['tx'], total_capacity)
         if outnum is None:
-            raise ValueError("no outnum found. capacity {} tx {}".format(total_capacity, tx))
+            raise ValueError("no outnum found. capacity {} tx {}".format(total_capacity, res['tx']))
 
-        return '{}x1x{}'.format(self.bitcoin.rpc.getblockcount(), outnum)
+        for i, txid in enumerate(self.bitcoin.rpc.getblock(blockid)['tx']):
+            if txid == res['txid']:
+                txnum = i
+
+        return '{}x{}x{}'.format(self.bitcoin.rpc.getblockcount(), txnum, outnum)
 
     def getactivechannels(self):
         return [c for c in self.rpc.listchannels()['channels'] if c['active']]
@@ -855,11 +880,14 @@ class LightningNode(object):
                                    announce=announce_channel,
                                    **kwargs)
         wait_for(lambda: res['txid'] in self.bitcoin.rpc.getrawmempool())
-        self.bitcoin.generate_block(1)
+        blockid = self.bitcoin.generate_block(1)[0]
 
-        # Hacky way to find our output.
-        scid = "{}x1x{}".format(self.bitcoin.rpc.getblockcount(),
-                                get_tx_p2wsh_outnum(self.bitcoin, res['tx'], amount))
+        for i, txid in enumerate(self.bitcoin.rpc.getblock(blockid)['tx']):
+            if txid == res['txid']:
+                txnum = i
+
+        scid = "{}x{}x{}".format(self.bitcoin.rpc.getblockcount(),
+                                 txnum, res['outnum'])
 
         if wait_for_active:
             self.wait_channel_active(scid)
@@ -968,7 +996,10 @@ class LightningNode(object):
         assert len(self.rpc.listpeers(dst_id).get('peers')) == 1
 
         # make an invoice
-        rhash = dst.rpc.invoice(amt, label, label)['payment_hash']
+        inv = dst.rpc.invoice(amt, label, label)
+        # FIXME: pre 0.10.1 invoice calls didn't have payment_secret field
+        psecret = dst.rpc.decodepay(inv['bolt11'])['payment_secret']
+        rhash = inv['payment_hash']
         invoices = dst.rpc.listinvoices(label)['invoices']
         assert len(invoices) == 1 and invoices[0]['status'] == 'unpaid'
 
@@ -980,7 +1011,7 @@ class LightningNode(object):
         }
 
         # sendpay is async now
-        self.rpc.sendpay([routestep], rhash)
+        self.rpc.sendpay([routestep], rhash, payment_secret=psecret)
         # wait for sendpay to comply
         result = self.rpc.waitsendpay(rhash)
         assert(result.get('status') == 'complete')
@@ -1009,9 +1040,9 @@ class LightningNode(object):
             params = r['params']
             if params == [2, 'CONSERVATIVE']:
                 feerate = feerates[0] * 4
-            elif params == [3, 'CONSERVATIVE']:
+            elif params == [6, 'ECONOMICAL']:
                 feerate = feerates[1] * 4
-            elif params == [4, 'ECONOMICAL']:
+            elif params == [12, 'ECONOMICAL']:
                 feerate = feerates[2] * 4
             elif params == [100, 'ECONOMICAL']:
                 feerate = feerates[3] * 4
@@ -1184,7 +1215,7 @@ class NodeFactory(object):
     """A factory to setup and start `lightningd` daemons.
     """
     def __init__(self, request, testname, bitcoind, executor, directory,
-                 db_provider, node_cls, throttler):
+                 db_provider, node_cls, throttler, jsonschemas):
         if request.node.get_closest_marker("slow_test") and SLOW_MACHINE:
             self.valgrind = False
         else:
@@ -1199,6 +1230,7 @@ class NodeFactory(object):
         self.db_provider = db_provider
         self.node_cls = node_cls
         self.throttler = throttler
+        self.jsonschemas = jsonschemas
 
     def split_options(self, opts):
         """Split node options from cli options
@@ -1216,7 +1248,8 @@ class NodeFactory(object):
             'random_hsm',
             'feerates',
             'wait_for_bitcoind_sync',
-            'allow_bad_gossip'
+            'allow_bad_gossip',
+            'start',
         ]
         node_opts = {k: v for k, v in opts.items() if k in node_opt_keys}
         cli_opts = {k: v for k, v in opts.items() if k not in node_opt_keys}
@@ -1276,6 +1309,7 @@ class NodeFactory(object):
         node = self.node_cls(
             node_id, lightning_dir, self.bitcoind, self.executor, self.valgrind, db=db,
             port=port, options=options, may_fail=may_fail or expect_fail,
+            jsonschemas=self.jsonschemas,
             **kwargs
         )
 
